"""
Testing suite for the null executable.
"""

import pytest
import argparse

from pytest_mock import MockerFixture

from null.null import Null, ERRORS

@pytest.fixture
def args():
    return argparse.Namespace(
        dump=False, exit_fail=False, exit_random=False, greeting=False, nap=False, print_error=False, verbose=False
    )


def test_print_error(args, caplog):
    args.print_error = True
    Null(args, True).execute()
    assert "ERROR: This is an error." in caplog.text


def test_print_greeting(args, caplog):
    args.greeting = True
    Null(args, True).execute()
    assert "Hello, world!" in caplog.text
    assert "And goodbye, world..." in caplog.text


def test_take_nap(args, caplog, mocker):
    mocker.patch("time.sleep", return_value=None)
    args.nap = True
    Null(args, True).execute()
    assert "Going to sleep..." in caplog.text
    assert "Waking up." in caplog.text


def test_exit_randomly(args, caplog):
    args.exit_random = True
    with pytest.raises(SystemExit) as e:
        Null(args, True).execute()
    assert "Exiting with status code" in caplog.text
    assert e.value.code in ERRORS.keys()


def test_exit_with_failure(args, caplog):
    args.exit_fail = True
    with pytest.raises(SystemExit) as e:
        Null(args, True).execute()
    assert ERRORS[1] in caplog.text
    assert e.value.code == 1
